#' @title Generate provenance for SUNSET command calls
#' @description Generate provenance information for command calls based on metaclipR function metalcipR.graph.Command()
#' @param graph Current provenance graph. 
#' @param package Package name
#' @param version Version name 
#' @param function Function name 
#' @param arg.list Argument list
#' @param origin.node.name List containing the name of the origin nodes to be linked to the command call
#' @return List containing updated graph.
#' @references https://earth.bsc.es/gitlab/es/sunset/-/wikis/home GitLab repository for SUNSET, a modular tool for climate forecast verification workflows.
#' @references This function is based on the semantics outlined in the ontology within the Metaclip Framework (available at \url{http://www.metaclip.org/}).

SunsetProv.Command<-function(graph, package, version,
                             fun, arg.list, origin.node.name) {
  #Check the inputs are correctly defined
  cmd.node.name <- paste0("Command_", randomName())
  # Argument and ArgumentValue
  time <- Sys.time()
  args <- paste(arg.list, collapse = " ")
  graph <- my_add_vertices(graph,
                           name = cmd.node.name,
                           className = "ds:Command",
                           label = fun,
                           attr = list("prov:value" = fun,
                                       "prov:atTime" =
                                         format(time, format = "%Y-%m-%d %Z")))

  # Add the edge linking the step with the command
  for (i in 1:length(origin.node.name)) {
    graph <- add_edges(graph,
                       c(
                         getNodeIndexbyName(
                                            graph, origin.node.name[i]),
                         getNodeIndexbyName(
                                            graph, cmd.node.name)),
                         label = "ds:hadCommandCall")
  }
  
  # Add Arguments (Recipe and Data)
  if ("Recipe" %in% arg.list) {
    recipe.nodename <- paste0("Argument_Recipe_", package)
    graph <- my_add_vertices(graph,
                             name = recipe.nodename,
                             className = "ds:Argument",
                             label = "Recipe",
                             attr =
                               list("dc:description" =
                                    "User-defined configuration file",
                                    "prov:AtLocation" = recipe$Run$output_dir))

    graph <- add_edges(graph,
                       c(
                         getNodeIndexbyName(
                                            graph, recipe.nodename),
                         getNodeIndexbyName(
                                            graph, cmd.node.name)),
                       label = "ds:usedArgument")
  }

  if ("Data" %in% arg.list) {
  recipe.nodename <- paste0("Argument_Data_", package)
  graph <- my_add_vertices(graph,
                           name = recipe.nodename,
                           className = "ds:Argument",
                           label = "Data",
                           attr =
                             list(
                               "dc:description" =
                               "s2dv_cube object",
                               "ds:referenceURL" =
                               "https://cran.r-project.org/
                                     web/packages/s2dv/index.html",
                               "dc:comment" = "output s2dv_cube object 
                                     from previous module"
                             ))

  graph <- add_edges(graph,
                      c(
                        getNodeIndexbyName(
                                          graph, recipe.nodename),
                        getNodeIndexbyName(
                                          graph, cmd.node.name)),
                      label = "ds:usedArgument")
}


  # Package
  pkg.nodename <- paste0("Package",randomName())
  graph <- my_add_vertices(graph = graph,
                            name = pkg.nodename,
                            className = "ds:Package",
                            label = package,
                            attr = list("ds:hasSoftwareVersion" = version))
  graph <- add_edges(graph, 
                      c(getNodeIndexbyName(graph, cmd.node.name),
                        getNodeIndexbyName(graph, pkg.nodename)),
                      label = "ds:fromPackage")
  return(graph)
}
